@@ -17,9 +17,10 @@
 """Remote dirs, repositories and branches."""
 
 import gzip
-from io import BytesIO
 import re
 
+from dulwich.refs import SymrefLoop
+
 from .. import (
     config,
     debug,
@@ -29,6 +30,7 @@ from .. import (
     ui,
     urlutils,
     )
+from ..controldir import BranchReferenceLoop
 from ..push import (
     PushResult,
     )
@@ -40,18 +42,19 @@ from ..errors import (
     InProcessTransport,
     InvalidRevisionId,
     LockContention,
-    NoSuchFile,
     NoSuchRevision,
     NoSuchTag,
     NotBranchError,
     NotLocalUrl,
     PermissionDenied,
     TransportError,
+    UnexpectedHttpStatus,
     UninitializableFormat,
     )
 from ..revision import NULL_REVISION
 from ..revisiontree import RevisionTree
 from ..transport import (
+    NoSuchFile,
     Transport,
     register_urlparse_netloc_protocol,
     )
@@ -76,7 +79,6 @@ from .dir import (
     )
 from .errors import (
     GitSmartRemoteNotSupported,
-    NoSuchRef,
     )
 from .mapping import (
     encode_git_path,
@@ -121,7 +123,6 @@ import os
 import select
 
 import urllib.parse as urlparse
-from urllib.parse import splituser
 
 # urlparse only supports a limited number of schemes by default
 register_urlparse_netloc_protocol('git')
@@ -172,6 +173,11 @@ class RemoteGitError(BzrError):
     _fmt = "Remote server error: %(msg)s"
 
 
+class ProtectedBranchHookDeclined(BzrError):
+
+    _fmt = "Protected branch hook declined"
+
+
 class HeadUpdateFailed(BzrError):
 
     _fmt = ("Unable to update remote HEAD branch. To update the master "
@@ -222,6 +228,14 @@ def parse_git_error(url, message):
         return TransportError('Host key verification failed')
     if message == '[Errno 104] Connection reset by peer':
         return ConnectionReset(message)
+    if message == 'The remote server unexpectedly closed the connection.':
+        return TransportError(message)
+    m = re.match(r'unexpected http resp ([0-9]+) for (.*)', message)
+    if m:
+        # TODO(jelmer): Have dulwich raise an exception and look at that instead?
+        return UnexpectedHttpStatus(path=m.group(2), code=int(m.group(1)), extra=message)
+    if message == 'protected branch hook declined':
+        return ProtectedBranchHookDeclined(msg=message)
     # Don't know, just return it to the user as-is
     return RemoteGitError(message)
 
@@ -234,7 +248,7 @@ def parse_git_hangup(url, e):
     """
     stderr_lines = getattr(e, 'stderr_lines', None)
     if not stderr_lines:
-        return e
+        return ConnectionReset('Connection closed early', e)
     if all(line.startswith(b'remote: ') for line in stderr_lines):
         stderr_lines = [
             line[len(b'remote: '):] for line in stderr_lines]
@@ -396,7 +410,7 @@ class DefaultProgressReporter(object):
         text = text.rstrip(b"\r\n")
         text = text.decode('utf-8', 'surrogateescape')
         if text.lower().startswith('error: '):
-            error = text[len(b'error: '):]
+            error = text[len('error: '):]
             self.errors.append(error)
             trace.show_error('git: %s', error)
         else:
@@ -414,7 +428,7 @@ class DefaultProgressReporter(object):
                     trace.note("%s", text)
 
 
-_LOCK_REF_ERROR_MATCHER = re.compile(b'cannot lock ref \'(.*)\': (.*)')
+_LOCK_REF_ERROR_MATCHER = re.compile('cannot lock ref \'(.*)\': (.*)')
 
 
 class RemoteGitDir(GitDir):
@@ -434,7 +448,9 @@ class RemoteGitDir(GitDir):
         return RemoteGitRepository
 
     def archive(self, format, committish, write_data, progress=None,
-                write_error=None, subdirs=None, prefix=None):
+                write_error=None, subdirs=None, prefix=None, recurse_nested=False):
+        if recurse_nested:
+            raise NotImplementedError('recurse_nested is not yet supported')
         if progress is None:
             pb = ui.ui_factory.nested_progress_bar()
             progress = DefaultProgressReporter(pb).progress
@@ -523,12 +539,12 @@ class RemoteGitDir(GitDir):
         refname = self._get_selected_ref(name, ref)
         if refname != b'HEAD' and refname in self.get_refs_container():
             raise AlreadyBranchError(self.user_url)
-        ref_chain, unused_sha = self.get_refs_container().follow(
+        ref_chain, sha = self.get_refs_container().follow(
             self._get_selected_ref(name))
         if ref_chain and ref_chain[0] == b'HEAD' and len(ref_chain) > 1:
             refname = ref_chain[1]
         repo = self.open_repository()
-        return RemoteGitBranch(self, repo, refname)
+        return RemoteGitBranch(self, repo, refname, sha)
 
     def destroy_branch(self, name=None):
         refname = self._get_selected_ref(name)
@@ -567,11 +583,25 @@ class RemoteGitDir(GitDir):
         return RemoteGitRepository(self)
 
     def get_branch_reference(self, name=None):
-        ref = branch_name_to_ref(name)
-        val = self.get_refs_container().read_ref(ref)
-        if val.startswith(SYMREF):
-            return val[len(SYMREF):]
-        return None
+        ref = self._get_selected_ref(name)
+        try:
+            ref_chain, unused_sha = self.get_refs_container().follow(ref)
+        except SymrefLoop:
+            raise BranchReferenceLoop(self)
+        if len(ref_chain) == 1:
+            return None
+        target_ref = ref_chain[1]
+        from .refs import ref_to_branch_name
+        try:
+            branch_name = ref_to_branch_name(target_ref)
+        except ValueError:
+            params = {'ref': urlutils.quote(target_ref.decode('utf-8'), '')}
+        else:
+            if branch_name != '':
+                params = {'branch': urlutils.quote(branch_name, '')}
+            else:
+                params = {}
+        return urlutils.join_segment_parameters(self.user_url.rstrip('/'), params)
 
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False, ref=None, possible_transports=None,
@@ -579,14 +609,16 @@ class RemoteGitDir(GitDir):
         repo = self.open_repository()
         ref = self._get_selected_ref(name, ref)
         try:
-            if not nascent_ok and ref not in self.get_refs_container():
-                raise NotBranchError(
-                    self.root_transport.base, controldir=self)
+            ref_chain, sha = self.get_refs_container().follow(ref)
+        except SymrefLoop:
+            raise BranchReferenceLoop(self)
         except NotGitRepository:
             raise NotBranchError(self.root_transport.base,
                                  controldir=self)
-        ref_chain, unused_sha = self.get_refs_container().follow(ref)
-        return RemoteGitBranch(self, repo, ref_chain[-1])
+        if not nascent_ok and sha is None:
+            raise NotBranchError(
+                self.root_transport.base, controldir=self)
+        return RemoteGitBranch(self, repo, ref_chain[-1], sha)
 
     def open_workingtree(self, recommend_upgrade=False):
         raise NotLocalUrl(self.transport.base)
@@ -779,6 +811,19 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         url = urlutils.strip_segment_parameters(str(url))
         super(BzrGitHttpClient, self).__init__(url, *args, **kwargs)
 
+    def archive(
+        self,
+        path,
+        committish,
+        write_data,
+        progress=None,
+        write_error=None,
+        format=None,
+        subdirs=None,
+        prefix=None,
+    ):
+        raise GitSmartRemoteNotSupported(self.archive, self)
+
     def _http_request(self, url, headers=None, data=None,
                       allow_compression=False):
         """Perform HTTP request.
@@ -786,7 +831,6 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         :param url: Request URL.
         :param headers: Optional custom headers to override defaults.
         :param data: Request data.
-        :param allow_compression: Allow GZipped communication.
         :return: Tuple (`response`, `read`), where response is an `urllib3`
             response object with additional `content_type` and
             `redirect_location` properties, and `read` is a consumable read
@@ -795,10 +839,6 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         if is_github_url(url):
             headers['User-agent'] = user_agent_for_github()
         headers["Pragma"] = "no-cache"
-        if allow_compression:
-            headers["Accept-Encoding"] = "gzip"
-        else:
-            headers["Accept-Encoding"] = "identity"
 
         response = self.transport.request(
             ('GET' if data is None else 'POST'),
@@ -812,15 +852,7 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
             raise GitProtocolError("unexpected http resp %d for %s" %
                                    (response.status, url))
 
-        # TODO: Optimization available by adding `preload_content=False` to the
-        # request and just passing the `read` method on instead of going via
-        # `BytesIO`, if we can guarantee that the entire response is consumed
-        # before issuing the next to still allow for connection reuse from the
-        # pool.
-        if response.getheader("Content-Encoding") == "gzip":
-            read = gzip.GzipFile(fileobj=BytesIO(response.read())).read
-        else:
-            read = response.read
+        read = response.read
 
         class WrapResponse(object):
 
@@ -902,7 +934,7 @@ class RemoteGitControlDirFormat(GitControlDirFormat):
 
 class GitRemoteRevisionTree(RevisionTree):
 
-    def archive(self, format, name, root=None, subdir=None, force_mtime=None):
+    def archive(self, format, name, root=None, subdir=None, force_mtime=None, recurse_nested=False):
         """Create an archive of this tree.
 
         :param format: Format name (e.g. 'tar')
@@ -911,6 +943,9 @@ class GitRemoteRevisionTree(RevisionTree):
         :param subdir: Subdirectory to export (or None)
         :return: Iterator over archive chunks
         """
+        if recurse_nested:
+            # TODO(jelmer): Parse .gitmodules from archive afterwards?
+            raise NotImplementedError('recurse_nested is not yet supported')
         commit = self._repository.lookup_bzr_revision_id(
             self.get_revision_id())[0]
         import tempfile
@@ -1038,9 +1073,9 @@ class RemoteGitTagDict(GitTags):
 
 class RemoteGitBranch(GitBranch):
 
-    def __init__(self, controldir, repository, name):
-        self._sha = None
-        super(RemoteGitBranch, self).__init__(controldir, repository, name,
+    def __init__(self, controldir, repository, ref, sha):
+        self._sha = sha
+        super(RemoteGitBranch, self).__init__(controldir, repository, ref,
                                               RemoteGitBranchFormat())
 
     def last_revision_info(self):
@@ -1062,14 +1097,6 @@ class RemoteGitBranch(GitBranch):
 
     @property
     def head(self):
-        if self._sha is not None:
-            return self._sha
-        refs = self.controldir.get_refs_container()
-        name = branch_name_to_ref(self.name)
-        try:
-            self._sha = refs[name]
-        except KeyError:
-            raise NoSuchRef(name, self.repository.user_url, refs)
         return self._sha
 
     def _synchronize_history(self, destination, revision_id):
