@@ -109,6 +109,7 @@ but helps protect against colliding host names.
 
 import os
 import time
+import yaml
 
 from . import (
     config,
@@ -116,21 +117,18 @@ from . import (
     errors,
     lock,
     osutils,
-    rio,
     ui,
     urlutils,
     )
 from .decorators import only_raises
 from .errors import (
     DirectoryNotEmpty,
-    FileExists,
     LockBreakMismatch,
     LockBroken,
     LockContention,
     LockCorrupt,
     LockFailed,
     LockNotHeld,
-    NoSuchFile,
     PathError,
     ResourceBusy,
     TransportError,
@@ -138,6 +136,10 @@ from .errors import (
 from .i18n import gettext
 from .osutils import format_delta, rand_chars, get_host_name
 from .trace import mutter, note
+from .transport import (
+    FileExists,
+    NoSuchFile,
+    )
 
 
 # XXX: At the moment there is no consideration of thread safety on LockDir
@@ -268,7 +270,7 @@ class LockDir(lock.Lock):
         if info is None:
             raise LockFailed(self, "lock was renamed into place, but "
                              "now is missing!")
-        if info.get('nonce') != self.nonce:
+        if info.nonce != self.nonce:
             self._trace("rename succeeded, "
                         "but lock is still held by someone else")
             raise LockContention(self)
@@ -328,7 +330,7 @@ class LockDir(lock.Lock):
             # After creating the lock directory, try again
             self.transport.mkdir(tmpname)
         info = LockHeldInfo.for_this_process(self.extra_holder_info)
-        self.nonce = info.get('nonce')
+        self.nonce = info.nonce
         # We use put_file_non_atomic because we just created a new unique
         # directory so we don't have to worry about files existing there.
         # We'll rename the whole directory into place to get atomic
@@ -445,7 +447,7 @@ class LockDir(lock.Lock):
         self.transport.delete(broken_info_path)
         self.transport.rmdir(tmpname)
         result = lock.LockResult(self.transport.abspath(self.path),
-                                 current_info.get('nonce'))
+                                 current_info.nonce)
         for hook in self.hooks['lock_broken']:
             hook(result)
         return result
@@ -500,7 +502,7 @@ class LockDir(lock.Lock):
         if info is None:
             # no lock there anymore!
             raise LockBroken(self)
-        if info.get('nonce') != self.nonce:
+        if info.nonce != self.nonce:
             # there is a lock, but not ours
             raise LockBroken(self)
 
@@ -687,7 +689,7 @@ class LockDir(lock.Lock):
                 # Lock isn't held
                 lock_token = None
             else:
-                lock_token = info.get('nonce')
+                lock_token = info.nonce
             if token != lock_token:
                 raise errors.TokenMismatch(token, lock_token)
             else:
@@ -713,8 +715,6 @@ class LockHeldInfo(object):
     read back by any process with access to the lockdir.  It can be used, for
     example, to tell the user who holds the lock, or to try to detect whether
     the lock holder is still alive.
-
-    Prior to bzr 2.4 a simple dict was used instead of an object.
     """
 
     def __init__(self, info_dict):
@@ -745,7 +745,7 @@ class LockHeldInfo(object):
             time_ago = '(unknown)'
         else:
             time_ago = format_delta(
-                time.time() - int(self.info_dict['start_time']))
+                time.time() - self.info_dict['start_time'])
         user = self.info_dict.get('user', '<unknown>')
         hostname = self.info_dict.get('hostname', '<unknown>')
         pid = self.info_dict.get('pid', '<unknown>')
@@ -755,6 +755,11 @@ class LockHeldInfo(object):
             pid=pid,
             time_ago=time_ago)
 
+    @property
+    def nonce(self):
+        nonce = self.get('nonce')
+        return nonce.encode('ascii') if nonce else None
+
     def get(self, field_name):
         """Return the contents of a field from the lock info, or None."""
         return self.info_dict.get(field_name)
@@ -765,9 +770,9 @@ class LockHeldInfo(object):
         """
         info = dict(
             hostname=get_host_name(),
-            pid=str(os.getpid()),
-            nonce=rand_chars(20).encode('ascii'),
-            start_time=str(int(time.time())),
+            pid=os.getpid(),
+            nonce=rand_chars(20),
+            start_time=int(time.time()),
             user=get_username_for_lock_info(),
             )
         if extra_holder_info is not None:
@@ -775,27 +780,24 @@ class LockHeldInfo(object):
         return cls(info)
 
     def to_bytes(self):
-        s = rio.Stanza(**self.info_dict)
-        return s.to_string()
+        return yaml.dump(self.info_dict).encode('utf-8')
 
     @classmethod
     def from_info_file_bytes(cls, info_file_bytes):
         """Construct from the contents of the held file."""
-        lines = osutils.split_lines(info_file_bytes)
         try:
-            stanza = rio.read_stanza(lines)
-        except ValueError as e:
+            ret = yaml.safe_load(info_file_bytes)
+        except yaml.reader.ReaderError as e:
+            lines = osutils.split_lines(info_file_bytes)
             mutter('Corrupt lock info file: %r', lines)
             raise LockCorrupt("could not parse lock info file: " + str(e),
                               lines)
-        if stanza is None:
+        if ret is None:
             # see bug 185013; we fairly often end up with the info file being
             # empty after an interruption; we could log a message here but
             # there may not be much we can say
             return cls({})
         else:
-            ret = stanza.as_dict()
-            ret['nonce'] = ret['nonce'].encode('ascii')
             return cls(ret)
 
     def __hash__(self):
@@ -814,7 +816,7 @@ class LockHeldInfo(object):
         """True if this process seems to be the current lock holder."""
         return (
             self.get('hostname') == get_host_name()
-            and self.get('pid') == str(os.getpid())
+            and self.get('pid') == os.getpid()
             and self.get('user') == get_username_for_lock_info())
 
     def is_lock_holder_known_dead(self):
