@@ -52,6 +52,7 @@ from io import BytesIO
 import itertools
 import re
 import sys
+from typing import Callable, List, Dict
 from warnings import (
     warn,
     )
@@ -65,7 +66,6 @@ from breezy import (
     diff,
     foreign,
     lazy_regex,
-    revision as _mod_revision,
     )
 from breezy.i18n import gettext, ngettext
 """)
@@ -73,8 +73,10 @@ from breezy.i18n import gettext, ngettext
 from . import (
     errors,
     registry,
+    revision as _mod_revision,
     revisionspec,
     trace,
+    transport as _mod_transport,
     )
 from .osutils import (
     format_date,
@@ -689,7 +691,7 @@ def _generate_all_revisions(branch, start_rev_id, end_rev_id, direction,
             # A merge was never detected so the lower revision limit can't
             # be nested down somewhere
             raise errors.CommandError(gettext('Start revision not found in'
-                                                 ' history of end revision.'))
+                                              ' history of end revision.'))
 
     # We exit the loop above because we encounter a revision with merges, from
     # this revision, we need to switch to _graph_view_revisions.
@@ -1214,7 +1216,7 @@ def _get_mainline_revs(branch, start_revision, end_revision):
         raise errors.CommandError(gettext('Logging revision 0 is invalid.'))
     if start_revno > end_revno:
         raise errors.CommandError(gettext("Start revision must be older "
-                                             "than the end revision."))
+                                          "than the end revision."))
 
     if end_revno < start_revno:
         return None, None, None, None
@@ -1933,7 +1935,7 @@ def author_list_committer(rev):
     return [rev.committer]
 
 
-author_list_registry = registry.Registry()
+author_list_registry = registry.Registry[str, Callable[[_mod_revision.Revision], List[str]]]()
 
 author_list_registry.register('all', author_list_all,
                               'All authors')
@@ -2186,11 +2188,11 @@ def _get_kind_for_file(tree, path):
     with tree.lock_read():
         try:
             return tree.stored_kind(path)
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return None
 
 
-properties_handler_registry = registry.Registry()
+properties_handler_registry = registry.Registry[str, Callable[[Dict[str, str]], Dict[str, str]]]()
 
 # Use the properties handlers to print out bug information if available
 
